#!/usr/bin/env python
# -*- encoding: UTF-8 -*-

# Copyright 2016 Eddie Antonio Santos <easantos@ualberta.ca>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

NEGATIVE_INFINITY = -float('inf')


def modification_time(filename):
    """
    Returns modification time of the given filename.
    Returns negative infinity if the file does not exist.
    """
    if not os.path.exists(filename):
        return NEGATIVE_INFINITY
    return os.stat(filename).st_mtime
